# -*- ruby -*-

#
# VFS for ruby - open anything with URI
# 
#   Copyright: Tatsuki Sugiura <sugi@nemui.org>
#   License: Ruby's original or GPL2
#   $LastChangedDate: 2003-10-13 10:05:51 +0900 (Mon, 13 Oct 2003) $
#   $LastChangedBy: sugi $
#

require 'vfs/dir'
require 'vfs/scheme/http/common'

module VFS;  module SchemeHandler;  module HTTP;
  class Dir < VFS::Dir
    include VFS::SchemeHandler::HTTP::Common
    
    def initialize(uri)
      super
      check_scheme(@uri)
      init_common_attr
    end

    def mkdir
      http_request(:mkcol)
      self
    end

    def rmdir
      http_request(:delete)
      self
    end
    alias :delete :rmdir
    alias :unlink :rmdir

    def entries
      @entries unless @entries == nil
      base = uri.path
      if capability? "PROPFIND" # with webdav ext
	http_request(:propfind, nil, {"Depth" => "1"}) { |rep|
	  @entries = []
	  rep.body.scan(/<D:href(?:\s+[^>]*)?>(.*?)<\/D:href>/m) { |e, |
	    # FIXME!!! MUST UNESCAPE XML ENTITY.
	    log("*BUG/NotInplementedYet* must unescape XML entity.", Log::WARNING)
	    e = e.delete("\n\r \t").sub(/^#{Regexp.quote(base)}/, '')
	    e == "" and next
	    @entries << e
	  }
	}
      else # not use webdav extention
	http_request { |rep|
	  @entries = []
	  rep.body.scan(/<a href=\"([^"]+)\"/) {|h, |
	    # fixme. VERY ad-hoc way...
	    h.sub!(/^\.\//, '')
	    h == ""           and next
	    h =~ /^\?/        and next
	    h =~ /^\//        and next # OK? realy??
	    h =~ /^\.\.\//    and next
	    h =~ /^[a-z+-]+:/ and next
	    # FIXME!!! MUST UNESCAPE HTML ENTITY.
	    log("*BUG/NotInplementedYet* must unescape HTML entity.", Log::WARNING)
	    if h =~ /\/(?!$)/
	      # Hmmmmmmmmmmm.
	      # fixme? User can open any URL resources as Dir.open,
	      # but script can't detect directory or not.
	      # may I raise exception like as Errno::ENOTDIR here.
	      raise Error::Unknown,
		"A directory entory ('#{h}') has '/'. This is bug..... or not."
	    end
	    @entries << h
	  }
	}
      end
      @entries
    end

  end
end; end; end

if $0 == __FILE__

#  uri = "http://localhost/~sugi/"
  uri = "http://sugi.nemui.org/pub/incomming/"
  f = VFS::SchemeHandler::HTTP::Dir.new(uri)
  p f.capability
  p f.entries

  uri = "http://sugi.nemui.org/pub/incomming/mkcoltest"
  n = VFS::SchemeHandler::HTTP::Dir.new(uri)
  p n.mkdir
  p f.entries
  p n.rmdir
end
