#
# VFS for ruby - open anything with URI
# 
#   Copyright: Tatsuki Sugiura <sugi@nemui.org>
#   License: Ruby's original or GPL2
#   $LastChangedDate: 2003-10-08 22:10:32 +0900 (Wed, 08 Oct 2003) $
#   $LastChangedBy: sugi $
#

require 'net/https'
require 'vfs/scheme/http/common'

module VFS;  module SchemeHandler;  module HTTPS;

  module Common;
    extend VFS::MethodGenerator
    include VFS::SchemeHandler::HTTP::Common

    @@ca_path = File.dirname(OpenSSL::Config::DEFAULT_CONFIG_FILE) + "/certs"
    unless File.exists?(@@ca_path) && File.directory?(@@ca_path)
      @@ca_path = nil
    end
#    class_var_accessor :ca_path

    def check_scheme(uri)
      unless uri.scheme == "https"
	raise Error::InvalidURI,
	  "Invalid HTTP URL '#{uri}' was speficied."
      end
    end

    def init_common_attr
      @capability   = []
      @prop_cache   = nil
      @prop_cache_t = 0
      @header       = nil
      @ca_path      = @@ca_path
      @verify_mode  = OpenSSL::SSL::VERIFY_PEER
    end
    attr_accessor :ca_path, :verify_mode

    def http_connect(t_uri)
      if ENV.member? "http_proxy"
	proxy = URI.parse(ENV["http_proxy"])
	proxy_host = proxy.host
	proxy_port = proxy.port
      end
      Net::HTTP.version_1_1
      log("Opening HTTP connection to #{uri.host}.", Log::INFO)
      h = Net::HTTP.new(t_uri.host, t_uri.port, proxy_host, proxy_port)
      h.use_ssl = true
      if ca_path
	h.verify_mode = verify_mode
	h.ca_path = ca_path
      end
      h
    end

    def cert
      http_connect.peer_cert
    end
    
  end; # Common

end; end; end
