# -*- ruby -*-

#
# VFS for ruby - open anything with URI
# 
#   Copyright: Tatsuki Sugiura <sugi@nemui.org>
#   License: Ruby's original or GPL2
#   $LastChangedDate: 2003-10-13 10:05:35 +0900 (Mon, 13 Oct 2003) $
#   $LastChangedBy: sugi $
#

require 'vfs/common'

module VFS
  class Tempbuf
    extend VFS::MethodGenerator
    @@string_max   = 1024 * 1024
    @@use_stringio = true
    class_var_reader   :use_stringio
    class_var_accessor :string_max

    def initialize(url)
      @io         = nil
      @path       = url
      @close_hook = nil
      begin
	require 'stringio'
	log("creating as StringIO.", Log::DEBUG)
	@io = StringIO.new
      rescue LoadError
	@@use_stringio = false
	switch_to_tempfile
      end
    end
    attr_reader   :path, :io
    attr_accessor :close_hook

    def kind_of?(c)
      if c == IO
	return true
      end
      super
    end

    def switch_to_tempfile
      require 'tempfile'
      log("Switching to Tempfile.", Log::INFO)
      new_io = Tempfile.new('ruby-vfs')
      if io && io.size > 0
	new_io << io.string 
	new_io.seek(io.pos)
      end
      @io = new_io
    end

    def <<(str)
      io << str
      if @@use_stringio && StringIO === io && @@string_max < io.size
	log("Buffer size was exceeded in StringIO", Log::INFO)
	switch_to_tempfile
      end
      io
    end

    def real_path
      io.path
    end

    def close
      if close_hook
	log("call close hook.", Log::DEBUG)
	io.flush
	close_hook.call
      end
      if @@use_stringio && StringIO === io
	io.rewind
      elsif Tempfile === io
	io.flush
	io.rewind
      else
	raise Error::Unknown, "unkown error! please bug report!!"
      end
    end

    def renew
      log("renew was called. dispose of old IO and all hooks.", Log::DEBUG)
      __send__ :close_hook=, nil
      begin
	io.close
      rescue
      end
      begin
	@io = StringIO.new
      rescue NameError
	switch_to_tempfile
      end
      self
    end

    def method_missing(meth, *args, &block)
      # FIXME; Is this slow?
      # instance methods must be generated for each io.instance_methods(fasle)
      if io.respond_to? meth
	io.send meth, *args, &block
      else
	raise NameError, "undefined method `#{meth}' for #{self.class}"
      end
    end

    private
    def log(*args)
      args[0] = "VFS(TempBuf): #{args[0]}"
      Log.log(*args)
    end
  end # Tempbuf

end # VFS
